@@ -676,6 +676,11 @@ ActionString actionstrings[] =
         action_moveresize,
         setup_action_resize
     },
+    {
+        "toggledockautohide",
+        action_toggle_dockautohide,
+        NULL
+    },
     {
         "toggleshowdesktop",
         action_toggle_show_desktop,
@@ -1084,8 +1089,6 @@ void action_lower(union ActionData *data)
     client_action_start(data);
     stacking_lower(CLIENT_AS_WINDOW(data->client.any.c));
     client_action_end(data);
-
-    focus_order_to_bottom(data->client.any.c);
 }
 
 void action_close(union ActionData *data)
@@ -1536,6 +1539,12 @@ void action_toggle_layer(union ActionData *data)
     client_action_end(data);
 }
 
+void action_toggle_dockautohide(union ActionData *data)
+{
+    config_dock_hide = !config_dock_hide;
+    dock_configure();
+}
+
 void action_toggle_show_desktop(union ActionData *data)
 {
     screen_show_desktop(!screen_showing_desktop);
