@@ -1,6 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    config.c for the Openbox window manager
+   Copyright (c) 2004        Mikael Magnusson
    Copyright (c) 2003        Ben Jansens
 
    This program is free software; you can redistribute it and/or modify
@@ -28,17 +29,22 @@ gboolean config_focus_new;
 gboolean config_focus_follow;
 guint    config_focus_delay;
 gboolean config_focus_raise;
+gboolean config_focus_last;
 
 ObPlacePolicy config_place_policy;
 
-gchar *config_theme;
+gchar   *config_theme;
+gboolean config_theme_keepborder;
 
 gchar *config_title_layout;
 
 gint    config_desktops_num;
 GSList *config_desktops_names;
+gint    config_screen_firstdesk;
 
-gboolean config_redraw_resize;
+gboolean config_resize_redraw;
+gint     config_resize_popup_show;
+gint     config_resize_popup_pos;
 
 ObStackingLayer config_dock_layer;
 gboolean        config_dock_floating;
@@ -57,6 +63,11 @@ guint config_keyboard_reset_state;
 gint config_mouse_threshold;
 gint config_mouse_dclicktime;
 
+gboolean config_menu_warppointer;
+gboolean config_menu_xorstyle;
+gboolean config_menu_hilightfirst;
+guint    config_menu_hide_delay;
+
 GSList *config_menu_files;
 
 gint config_resist_win;
@@ -204,6 +215,8 @@ static void parse_focus(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
         config_focus_delay = parse_int(doc, n) * 1000;
     if ((n = parse_find_node("raiseOnFocus", node)))
         config_focus_raise = parse_bool(doc, n);
+    if ((n = parse_find_node("focusLast", node)))
+        config_focus_last = parse_bool(doc, n);
 }
 
 static void parse_placement(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
@@ -237,6 +250,8 @@ static void parse_theme(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
         g_free(config_title_layout);
         config_title_layout = parse_string(doc, n);
     }
+    if ((n = parse_find_node("keepBorder", node)))
+        config_theme_keepborder = parse_bool(doc, n);
 }
 
 static void parse_desktops(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
@@ -247,10 +262,15 @@ static void parse_desktops(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
     node = node->children;
     
     if ((n = parse_find_node("number", node))) {
-        guint d = parse_int(doc, n);
+        gint d = parse_int(doc, n);
         if (d > 0)
             config_desktops_num = d;
     }
+    if ((n = parse_find_node("firstdesk", node))) {
+        gint d = parse_int(doc, n);
+        if (d > 0)
+            config_screen_firstdesk = d;
+    }
     if ((n = parse_find_node("names", node))) {
         GSList *it;
         xmlNodePtr nname;
@@ -277,7 +297,23 @@ static void parse_resize(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
     node = node->children;
     
     if ((n = parse_find_node("drawContents", node)))
-        config_redraw_resize = parse_bool(doc, n);
+        config_resize_redraw = parse_bool(doc, n);
+    if ((n = parse_find_node("popupShow", node))) {
+        config_resize_popup_show = parse_int(doc, n);
+        if (parse_contains("Always", doc, n))
+            config_resize_popup_show = 2;
+        else if (parse_contains("Never", doc, n))
+            config_resize_popup_show = 0;
+        else if (parse_contains("Nonpixel", doc, n))
+            config_resize_popup_show = 1;
+    }
+    if ((n = parse_find_node("popupPosition", node))) {
+        config_resize_popup_pos = parse_int(doc, n);
+        if (parse_contains("Top", doc, n))
+            config_resize_popup_pos = 1;
+        else if (parse_contains("Center", doc, n))
+            config_resize_popup_pos = 0;
+    }
 }
 
 static void parse_dock(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
@@ -355,6 +391,7 @@ static void parse_dock(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
 static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
                        gpointer d)
 {
+    xmlNodePtr n;
     for (node = node->children; node; node = node->next) {
         if (!xmlStrcasecmp(node->name, (const xmlChar*) "file")) {
             gchar *c;
@@ -364,6 +401,14 @@ static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
                                                parse_expand_tilde(c));
             g_free(c);
         }
+        if ((n = parse_find_node("warpPointer", node)))
+            config_menu_warppointer = parse_bool(doc, n);
+        if ((n = parse_find_node("xorStyle", node)))
+            config_menu_xorstyle = parse_bool(doc, n);
+        if ((n = parse_find_node("hilightFirst", node)))
+            config_menu_hilightfirst = parse_bool(doc, n);
+        if ((n = parse_find_node("hideDelay", node)))
+            config_menu_hide_delay = parse_int(doc, n);
     }
 }
    
@@ -488,6 +533,7 @@ void config_startup(ObParseInst *i)
     config_focus_follow = FALSE;
     config_focus_delay = 0;
     config_focus_raise = FALSE;
+    config_focus_last = FALSE;
 
     parse_register(i, "focus", parse_focus, NULL);
 
@@ -498,15 +544,19 @@ void config_startup(ObParseInst *i)
     config_theme = NULL;
 
     config_title_layout = g_strdup("NLIMC");
+    config_theme_keepborder = TRUE;
 
     parse_register(i, "theme", parse_theme, NULL);
 
     config_desktops_num = 4;
+    config_screen_firstdesk = 1;
     config_desktops_names = NULL;
 
     parse_register(i, "desktops", parse_desktops, NULL);
 
-    config_redraw_resize = TRUE;
+    config_resize_redraw = TRUE;
+    config_resize_popup_show = 1; /* nonpixel increments */
+    config_resize_popup_pos = 0;  /* center of client */
 
     parse_register(i, "resize", parse_resize, NULL);
 
@@ -542,6 +592,10 @@ void config_startup(ObParseInst *i)
 
     parse_register(i, "resistance", parse_resistance, NULL);
 
+    config_menu_warppointer = TRUE;
+    config_menu_xorstyle = TRUE;
+    config_menu_hilightfirst = TRUE;
+    config_menu_hide_delay = 250;
     config_menu_files = NULL;
 
     parse_register(i, "menu", parse_menu, NULL);
