@@ -1,6 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    config.h for the Openbox window manager
+   Copyright (c) 2004        Mikael Magnusson
    Copyright (c) 2003        Ben Jansens
 
    This program is free software; you can redistribute it and/or modify
@@ -36,12 +37,19 @@ extern guint    config_focus_delay;
 /*! If windows should automatically be raised when they are focused in
  focus follows mouse */
 extern gboolean config_focus_raise;
+/*! Focus the last focused window, not under the mouse, in follow mouse mode */
+extern gboolean config_focus_last;
 
 extern ObPlacePolicy config_place_policy;
 
 /*! When true windows' contents are refreshed while they are resized; otherwise
   they are not updated until the resize is complete */
-extern gboolean config_redraw_resize;
+extern gboolean config_resize_redraw;
+/*! show move/resize popups? 0 = no, 1 = always, 2 = only
+  resizing !1 increments */
+extern gint config_resize_popup_show;
+/*! where to show the popup, currently above the window or centered */
+extern gint config_resize_popup_pos;
 
 /*! The stacking layer the dock will reside in */
 extern ObStackingLayer config_dock_layer;
@@ -69,11 +77,15 @@ extern guint config_dock_app_move_modifiers;
 /* The name of the theme */
 extern gchar *config_theme;
 
+/* Show the onepixel border after toggleDecor */
+extern gboolean config_theme_keepborder;
 /* Titlebar button layout */
 extern gchar *config_title_layout;
 
 /*! The number of desktops */
 extern gint config_desktops_num;
+/*! Desktop to start on, put 5 to start in the center of a 3x3 grid */
+extern gint config_screen_firstdesk;
 /*! Names for the desktops */
 extern GSList *config_desktops_names;
 
@@ -93,6 +105,14 @@ extern gint config_resist_win;
 /*! Number of pixels to resist while crossing a screen's edge */
 extern gint config_resist_edge;
 
+/*! Warp near edge on menu? */
+extern gboolean config_menu_warppointer;
+/*! make menus jump around a lot */
+extern gboolean config_menu_xorstyle;
+/*! hilight first entry in menu */
+extern gboolean config_menu_hilightfirst;
+/*! delay for hiding menu when opening */
+extern guint    config_menu_hide_delay;
 /*! User-specified menu files */
 extern GSList *config_menu_files;
 
