@@ -1,6 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    mainloop.c for the Openbox window manager
+   Copyright (c) 2004        Mikael Magnusson
    Copyright (c) 2003        Ben Jansens
 
    This program is free software; you can redistribute it and/or modify
@@ -121,7 +122,6 @@ struct _ObMainLoopXHandlerType
     ObMainLoop *loop;
     gpointer data;
     ObMainLoopXHandler func;
-    ObMainLoopXDoneHandler done_func;
     GDestroyNotify destroy;
 };
 
@@ -294,7 +294,7 @@ void ob_main_loop_run(ObMainLoop *loop)
             for (i = 0; i < NUM_SIGNALS; ++i) {
                 while (loop->signals_fired[i]) {
                     for (it = loop->signal_handlers[i];
-                         it; it = g_slist_next(it)) {
+                            it; it = g_slist_next(it)) {
                         ObMainLoopSignalHandlerType *h = it->data;
                         h->func(i, h->data);
                     }
@@ -313,60 +313,51 @@ void ob_main_loop_run(ObMainLoop *loop)
                     h->func(&e, h->data);
                 }
             } while (XPending(loop->display));
-        } else {
-            for (it = loop->x_handlers; it; it = g_slist_next(it)) {
-                ObMainLoopXHandlerType *h = it->data;
-                if (h->done_func)
-                    h->done_func(h->data);
-            }
+        } else if (loop->action_queue) {
+            /* only fire off one action at a time, then go back for more
+               X events, since the action might cause some X events (like
+               FocusIn :) */
 
-            if (loop->action_queue) {
-                /* only fire off one action at a time, then go back for more
-                   X events, since the action might cause some X events (like
-                   FocusIn :) */
-
-                do {
-                    act = loop->action_queue->data;
-                    if (act->data.any.client_action ==
-                        OB_CLIENT_ACTION_ALWAYS &&
-                        !act->data.any.c)
-                    {
-                        loop->action_queue =
-                            g_slist_delete_link(loop->action_queue,
-                                                loop->action_queue);
-                        action_unref(act);
-                        act = NULL;
-                    }
-                } while (!act && loop->action_queue);
-
-                if  (act) {
-                    act->func(&act->data);
+            do {
+                act = loop->action_queue->data;
+                if (act->data.any.client_action == OB_CLIENT_ACTION_ALWAYS &&
+                    !act->data.any.c)
+                {
                     loop->action_queue =
                         g_slist_delete_link(loop->action_queue,
-                                            loop->action_queue);
+                                loop->action_queue);
                     action_unref(act);
+                    act = NULL;
                 }
-            } else {
-                /* this only runs if there were no x events received */
+            } while (!act && loop->action_queue);
 
-                timer_dispatch(loop, (GTimeVal**)&wait);
-
-                selset = loop->fd_set;
-                /* there is a small race condition here. if a signal occurs
-                   between this if() and the select() then we will not process
-                   the signal until 'wait' expires. possible solutions include
-                   using GStaticMutex, and having the signal handler set 'wait'
-                   to 0 */
-                if (!loop->signal_fired)
-                    select(loop->fd_max + 1, &selset, NULL, NULL, wait);
-
-                /* handle the X events with highest prioirity */
-                if (FD_ISSET(loop->fd_x, &selset))
-                    continue;
-
-                g_hash_table_foreach(loop->fd_handlers,
-                                     fd_handle_foreach, &selset);
+            if  (act) {
+                act->func(&act->data);
+                loop->action_queue =
+                    g_slist_delete_link(loop->action_queue,
+                                        loop->action_queue);
+                action_unref(act);
             }
+        } else {
+            /* this only runs if there were no x events received */
+
+            timer_dispatch(loop, (GTimeVal**)&wait);
+
+            selset = loop->fd_set;
+            /* there is a small race condition here. if a signal occurs
+               between this if() and the select() then we will not process
+               the signal until 'wait' expires. possible solutions include
+               using GStaticMutex, and having the signal handler set 'wait'
+               to 0 */
+            if (!loop->signal_fired)
+                select(loop->fd_max + 1, &selset, NULL, NULL, wait);
+
+            /* handle the X events with highest prioirity */
+            if (FD_ISSET(loop->fd_x, &selset))
+                continue;
+
+            g_hash_table_foreach(loop->fd_handlers,
+                                 fd_handle_foreach, &selset);
         }
     }
 
@@ -384,7 +375,6 @@ void ob_main_loop_exit(ObMainLoop *loop)
 
 void ob_main_loop_x_add(ObMainLoop *loop,
                         ObMainLoopXHandler handler,
-                        ObMainLoopXDoneHandler done_handler,
                         gpointer data,
                         GDestroyNotify notify)
 {
@@ -393,7 +383,6 @@ void ob_main_loop_x_add(ObMainLoop *loop,
     h = g_new(ObMainLoopXHandlerType, 1);
     h->loop = loop;
     h->func = handler;
-    h->done_func = done_handler;
     h->data = data;
     h->destroy = notify;
     loop->x_handlers = g_slist_prepend(loop->x_handlers, h);
@@ -679,7 +668,7 @@ static void timer_dispatch(ObMainLoop *loop, GTimeVal **wait)
             g_free(curr);
             continue;
         }
-	
+
         /* the queue is sorted, so if this timer shouldn't fire, none are 
            ready */
         if (timecompare(&NEAREST_TIMEOUT(loop), &loop->now) < 0)
