@@ -1,6 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    popup.c for the Openbox window manager
+   Copyright (c) 2004        Mikael Magnusson
    Copyright (c) 2003        Ben Jansens
 
    This program is free software; you can redistribute it and/or modify
@@ -105,6 +106,12 @@ void popup_show(ObPopup *self, gchar *text)
     gint x, y, w, h;
     gint textw, texth;
     gint iconw;
+    Rect *area; /* won't go outside this */
+
+    area = screen_physical_area();          /* XXX this should work quite
+                                               good, someone with xinerama,
+                                               and different resolutions on
+                                               screens? */
 
     RrMargins(self->a_bg, &l, &t, &r, &b);
 
@@ -170,6 +177,9 @@ void popup_show(ObPopup *self, gchar *text)
         break;
     }
 
+    x=MAX(MIN(x, area->width-w),0);
+    y=MAX(MIN(y, area->height-h),0);
+
     /* set the windows/appearances up */
     XMoveResizeWindow(ob_display, self->bg, x, y, w, h);
 
